
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cell.h"
#include "rif.h"
#include "t_s.h"

extern void	Error(),RangeList();

#ifndef	M_PI
#define M_PI	3.14159265358979323846
#endif

typedef struct List {
		int		hi,vi,hj,vj,hk,vk;
		struct List	*next;
	} List;

static List	**ptr;

static void	Store(hi,vi,hj,vj,hk,vk)
int		hi,vi,hj,vj,hk,vk;
{
	List	*list;

	if ((*ptr=list=(List *)malloc(sizeof(List)))==NULL)
	    Error("list memory allocation error.");

	list->hi=hi; list->vi=vi;
	list->hj=hj; list->vj=vj;
	list->hk=hk; list->vk=vk; ptr=(&(list->next));
}

#define PV	(size_t)((Cell)(~0))

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	Nh,Nv,h1,h2,v1,v2,v,h,N,M,m,n,n1,n2,n3;
	Cell	**cell;
	char	occupied[PV+1],*argv_dr,**back;
	double	dr,h0,v0,W,x,y,w,R,*p,t,rh,rv,r0,sh,sv,s0,
		ri,si,rj,sj,rk,sk,r1,s1,r2,s2,r3,s3,S,ux=1.0,uy=1.0;
	List	*next,*list;
	FILE	*file;

	if (argc<5 || argc>8)
	    Error("usage : i_s_sg TIFF rangeList {ux uy} dr {M} SG");

	ReadImageFile(argv[1],&Nh,&Nv,NULL,&cell,NULL);

	RangeList(argv[2],PV,occupied);

	if (argc==5 || argc==6)
	    dr=atof(argv_dr=argv[3]);
	else {
	    if ((ux=atof(argv[3]))<=0.0 ||
		(uy=atof(argv[4]))<=0.0) Error("bad pixel size unit.");

	    dr=atof(argv_dr=argv[5]);
	}
	if (dr<=0.0) Error("bad detector interval.");

	InitTriangulation(Nh,Nv,&back);

	h1=Nh; h2=(-1); h0=(double)Nh/2.0;
	v1=Nv; v2=(-1); v0=(double)Nv/2.0; W=0.0;

	for (v=0; v<Nv; v++) {
	    for (h=0; h<Nh; h++)
		if (occupied[cell[v][h]]) {
		    back[v][h]=0;

		    if (h<h1) h1=h; if (h>h2) h2=h;
		    if (v<v1) v1=v; if (v>v2) v2=v;

		    x=ux*(fabs((double)h+0.5-h0)+0.5);
		    y=uy*(fabs(v0-(double)v-0.5)+0.5); if ((w=x*x+y*y)>W) W=w;
		}
		else
		    back[v][h]=1;

	    free(cell[v]);
	}
	free(cell);

	N=(int)ceil(2.0*sqrt(W)/dr); if (N==0) Error("no occupied pixel.");

	R=(double)N/2.0;

	if (argc==5 || argc==7)
	    M=(int)ceil(M_PI*R);
	else
	    if ((M=atoi(argv[argc-2]))<=0) Error("bad number of projections.");

	ptr=(&next); Triangulation(h1,v1,h2,v2,back,Store); *ptr=NULL;

	TermTriangulation(back);

	if ((p=(double *)malloc(sizeof(double)*(size_t)(N+2)))==NULL)
	    Error("sinogram memory allocation error.");

	++p;

	if (argv[argc-1][0]=='-')
	    file=stdout;
	else
	    if ((file=fopen(argv[argc-1],"wb"))==NULL) Error("file not open.");

	(void)fprintf(stderr,"%d\t%d\t%s\t%lg\n",N,M,argv_dr,(double)(1-N)/2.0);

	(void)fprintf(file,"%d\t%d\n",N,M);
	for (m=0; m<M; m++) {
	    for (n=0; n<N; n++) p[n]=0.0;

	    t=M_PI*(double)m/(double)M; x=cos(t); y=sin(t);
	    rh=  x *ux/dr; rv=y*(-uy)/dr; r0=rh*h0+rv*v0-R;
	    sh=(-y)*ux   ; sv=x*(-uy)   ; s0=sh*h0+sv*v0;

	    for (list=next; list!=NULL; list=list->next) {
		ri=rh*(double)list->hi+rv*(double)list->vi-r0;
		si=sh*(double)list->hi+sv*(double)list->vi-s0;
		rj=rh*(double)list->hj+rv*(double)list->vj-r0;
		sj=sh*(double)list->hj+sv*(double)list->vj-s0;
		rk=rh*(double)list->hk+rv*(double)list->vk-r0;
		sk=sh*(double)list->hk+sv*(double)list->vk-s0;
		if (ri<rj)
		    if (rj<rk) {
			r1=ri; s1=si; r2=rj; s2=sj; r3=rk; s3=sk;
		    }
		    else if (ri<rk) {
			r1=ri; s1=si; r2=rk; s2=sk; r3=rj; s3=sj;
		    }
		    else {
			r1=rk; s1=sk; r2=ri; s2=si; r3=rj; s3=sj;
		    }
		else
		    if (ri<rk) {
			r1=rj; s1=sj; r2=ri; s2=si; r3=rk; s3=sk;
		    }
		    else if (rj<rk) {
			r1=rj; s1=sj; r2=rk; s2=sk; r3=ri; s3=si;
		    }
		    else {
			r1=rk; s1=sk; r2=rj; s2=sj; r3=ri; s3=si;
		    }

		n1=(int)floor(r1);
		n2=(int)floor(r2);
		n3=(int)floor(r3); S=fabs(s2-s1-(s3-s1)*(r2-r1)/(r3-r1))/2.0;
		if (n1==n2)
		    p[n1]+=((r2-r1)*S);
		else {
		    n=n1+1; t=S/(r2-r1); w=(double)n-r1; W=t*w; p[n1]+=(w*W);
		    while (n<n2) {
			w=W; W+=t; p[n++]+=(w+W);
		    }
		    p[n2]+=((r2-(double)n2)*(W+S));
		}
		if (n2==n3)
		    p[n2]+=((r3-r2)*S);
		else {
		    n=n2+1; t=S/(r3-r2);
		    w=(double)n; W=t*(r3-w); p[n2]+=((w-r2)*(S+W));
		    while (n<n3) {
			w=W; W-=t; p[n++]+=(w+W);
		    }
		    p[n3]+=((r3-(double)n3)*W);
		}
	    }
	    (void)fwrite(p,sizeof(double),(size_t)N,file);
	}
	if (file!=stdout) (void)fclose(file);

	return 0;
}
