
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef	Filter
#define Filter	Chesler
#else
extern double	Filter(int i);
#endif

static void	Error(char *msg)
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

double	Ramachandran(int i)
{
	return (i==0)?1.0/4.0:(i&1)?-1.0/(M_PI*M_PI*(double)i*(double)i):0.0;
}

double	Shepp(int i)
{
	return 2.0/(M_PI*M_PI*(1.0-4.0*(double)i*(double)i));
}

double	Chesler(int i)
{
	if (i==0)
	    return 1.0/8.0-1.0/(2.0*M_PI*M_PI);
	else if (i==1 || i==(-1))
	    return 1.0/16.0-1.0/(2.0*M_PI*M_PI);
	else if (i&1)
	    return -1.0/(2.0*M_PI*M_PI*(double)i*(double)i);
{
	double	i2=(double)i*(double)i,i21=i2-1.0;

	return -(i2+1.0)/(2.0*M_PI*M_PI*i21*i21);
}}

int	main(int argc,char **argv)
{
	FILE	*file;
	int	L,l,M,n,m;
	char	*dre="data read error.",
		*dwe="data write error.";
	double	*P,*Q,dt,dr,q,r;

	if (argc!=4 && argc!=5)
	    Error("usage : rso_cbp projection {views} interval CT-value");

	if (argv[1][0]=='-')
	    file=stdin;
	else
	    if ((file=fopen(argv[1],"r"))==NULL) Error("file not found.");

	if (fscanf(file,"%d",&L)!=1 || L<=0) Error(dre);

	if ((P=(double *)malloc(sizeof(*P)*(size_t)L))==NULL ||
	    (Q=(double *)malloc(sizeof(*Q)*(size_t)L))==NULL)
	    Error("memory allocation error.");

	for (l=0; l<L; l++) if (fscanf(file,"%lf",P+l)!=1) Error(dre);

	if (file!=stdin) (void)fclose(file);

	if (argc==4)
	    M=(int)(M_PI*(double)L);
	else
	    if ((M=atoi(argv[2]))<=0) Error("bad number of views.");

	dt=M_PI/(double)M;

	if ((dr=atof(argv[argc-2]))<=0.0) Error("bad detector interval.");

	if (argv[argc-1][0]=='-')
	    file=stdout;
	else
	    if ((file=fopen(argv[argc-1],"wb"))==NULL) Error("file not open.");

	if (fprintf(file,"%d\n",L)<0) Error(dwe);

	for (l=0; l<L; l++) {
	    q=Filter(l)*P[0];
	    for (n=1; n<L; n++) q+=((Filter(l-n)+Filter(l+n))*P[n]);
	    Q[l]=q;
	}
	for (l=0; l<L; l++) {
	    q=Q[l];
	    for (m=1; m<M; m++) {
		r=fabs((double)l*cos(dt*(double)m));
		n=(int)r;
		q+=((r-(double)n)*(Q[n+1]-Q[n])+Q[n]);
	    }
	    if (fprintf(file,"%lf\n",q*dt/dr)<0) Error(dwe);
	}
	if (file!=stdout) (void)fclose(file);

	return 0;
}
